//
//    Copyright 2004 - 2007 Thomas C. McDermott, N5EG
//    This file is part of VNAR - the Vector Network Analyzer program.
//
//    VNAR is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    VNAR is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with VNAR, if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#pragma once

// Utility routines to convert VNA measurements into other
// useful types and coordinates.
// 


#include "stdafx.h"
#include "DirectionalCoupler.h"
#include "Detector.h"
#include "DataDisplay.h"
#using <mscorlib.dll>
#using <System.dll>
using namespace System;
using namespace System::IO;
using namespace System::Windows::Forms;


	// Convert Magnitude to rectangular Y display coordinate
int ToDisplayRectMag(double magnitude, int height, float dbScaleFactor, int refLevel);

	// Convert Phase to rectangular Y display coordinate
int ToDisplayRectPhs(double phase, int height);

	// Convert Group Delay to rectangular Y display coordinate
int ToDisplayRectGD(double groupdelay, int height, int scaleFactor);

	// Convert Magnitude and Phase to polar (X,Y) display coordinates
void ToDisplayPolar(double magnitude, double phase, int polarRad, int xoffset, int yoffset, int& X, int& Y);

	// Convert Resistance portion of S11 to Y display coordinate
int ToDisplayRectR(float resistance, int scale, int height);

	// Convert Reactance portion of S11 to Y display coordinate
int ToDisplayRectjX(float reactance, int scale, int height);

	// Compensate for ModAmpbuffer amplifer phase distortions
double BufferAmpPhaseComp(double phaseDegrees, int Frequency);

	/// Frequency Grid Class
__gc public class FrequencyGrid
{
private:
	int FrequencyIndex __gc[];			///< Grid of frequencies to measure
	int startFreq;						///< Starting (lowest) Frequency of grid
	int stopFreq;						///< Stopping (highest) Frequency of grid
	int indexer;						///< internal variable to index through grid
	int points;							///< Number of points in the frequency grid
	double delta;						///< Frequency separation between points in the grid
	int ferror;							///< Frequency error of internal crystal at 100 Mhz (from calibration)

public:
	FrequencyGrid(int numPoints);		// Constructor, allocate array
	void SetStartF(int start);			// Set start frequency of grid
	void SetStopF(int stop);			// Set stop frequency of grid
	int Frequency(int gridpoint);		// convert gridpoint to it's frequency
	long long int DDS(int Frequency);	// Derive DDS divisor value from Frequency
	int GridPoint(int Frequency);		// convert Frequency to gridpoint
	__property int get_Points();		// get number of points in grid
	__property int get_StartF();		// get start frequency of grid
	__property int get_StopF();			// get stop frequency of grid
	__property int get_Ferror();		// get Frequency error
	__property void set_Ferror(int FreqError);	// set Frequency error

private:
	void Build(void);
};


    /// Object to hold Calibration data
__gc public class CalDataSet
{
public:
	/// Holds raw and computed calibration data

	Detector* RxDet, * TxDet;		/// Holds detector constants
	DirectionalCoupler* DirCoupler;		/// Holds Directional coupler error model

	double EdReal __gc[], EdImag __gc[], EsReal __gc[], EsImag __gc[];
	double EtReal __gc[], EtImag __gc[], ThReal __gc[], ThImag __gc[];
	double S11shortReal __gc[], S11shortImag __gc[];
	double S11openReal __gc[], S11openImag __gc[];
	double S11termReal __gc[], S11termImag __gc[];

	int FreqError;					// Internal Crystal Frequency Error
	double reflTimeDelayEquivalent; // Equivalent time delay of reflection cable go and return (for RefExtn - not enabled)

	CalDataSet(String  __gc *StartUpDir);
		// resolve reflected measured data set to Magnitude and Phase
void ResolveReflPolar(MeasurementSet * dataPoint, int Freq, double& rmag, double& rphs, bool CouplerComp);
	// resolve transmitted measured data set to Magnitude and Phase
void ResolveTranPolar(MeasurementSet * dataPoint, int Freq, double& rmag, double& rphs);
};


// Derive error terms from cal measurements
void CalToErrorTerms(CalDataSet *Cal);

// Convert measured S11 into actual S11 via calibration
void CorrectS11(CalDataSet *Cal, int Frequency, bool ReflExtn, double measmag, double measphs, double& rsltmag, double& rsltphs);

// Convert measured S21 into actual S21 via calibration
void CorrectS21(CalDataSet *Cal, int Frequency, double measmag, double measphs, double& rsltmag, double& rsltphs);

// Load calibration data from a previously saved file
bool LoadCalDataSet(OpenFileDialog *infile, CalDataSet *Cal);

// Save newly acquired calibration data to a file
void SaveCalDataSet(SaveFileDialog *outfile, CalDataSet *Cal);

// Convert Log Tx Level to linear value needed by DAC register
unsigned short TxLevLinear(int tranLevel);

// Convert linear DAC level to dBm.  4095 = 0.0 dBm.
float dBFromLinear(int DAClevel);

// Convert Return Loss (S11 mag) to SWR, then to vertical display coordinate
int ToDisplayAsSWR(double mag, int height, int scaledB);

// Export S Parameters to a file
void ExportSParams(int format,  FrequencyGrid * FG,	double S11Mag __gc[],  double S11Phs __gc[],
				double S21Mag __gc[],  double S21Phs __gc[], double S22Mag __gc[],  double S22Phs __gc[],
				double S12Mag __gc[],  double S12Phs __gc[]);

// Store S Parameters to temporary storage
void StoreSParams(bool calmode, bool ReflExtn, FrequencyGrid * FG, CalDataSet * CalData,
					MeasurementSet * __gc[], double reflMag __gc[],  double reflPhs __gc[],
					double tranMag __gc[], double tranPhs __gc[]);

// Linear interpolation of Xval between Xlow and Xhigh yielding Y result between Ylow and Yhi
double Interpolate(int Xval, int Xlow, double Ylow, int Xhi, double Yhi);
//double Interpolate(int Xval, int Xlow, int Ylow, int Xhi, int Yhi);
//double Interpolate(float Xval, float Xlow, float Ylow, float Xhi, float Yhi);

// Median filtering routine.  Input is 7 measurements, returns the median value. Three overloads.
// Don't filter if the error is smaller than epsilon.
int Median7(unsigned short data __gc[], int index);
int Median7(int data __gc[]);
double Median7(double data __gc[], int, double epsilon = 0.0);
// Range routine. Determine range of middle 5 samples in a group of 7 samples.
int Range											/// \return Max-min range of the readings spanning 5 closest samples
				(unsigned short data __gc[],	///< Array of all measurement samples, one per frequency
				int index						///< Look at the 7 samples surrounding (and including) this index
				);
int ExpectedValue									/// \return Actual raw reading if it's close to
													/// the expected reading, Else the median of the
													/// 7 adjacent samples that include the reading.
				 (unsigned short data __gc[],	///< Array of all measurement samples, one per frequency
				  int index,					///< The sample we wish to find the expected value of
				  bool mode = false			///< Switch that allows for phase detector saturation.. true=phase, false=amplitude
				  );

// Deglitch an entire array of raw VNA readings using ExpectedValue
void DeGlitch(int raw __gc[], int count, bool mode);
void DeGlitch(MeasurementSet *dataSet __gc[] , int count);
// Smooth a phase error table
void SmoothPhaseTable(double table __gc[]);
// DrawLine routines that checks and bounds points to display rectangle
void DrawLineBound(System::Drawing::Graphics *, System::Drawing::Rectangle, System::Drawing::Pen *, int, int, int, int);
void DrawLineBound(System::Drawing::Graphics *, System::Drawing::Rectangle, System::Drawing::Pen *, System::Drawing::Point, System::Drawing::Point);
// Add 2 polar numbers
void AddPolar(double& linearMag, double& phaseDegr, double linearMagadd, double phaseDegrAdd);
// Convert MeasurementDelay string to target's loop count
int MeasureDelayStringToCount(String * value);
// Convert Detector Magnitude Calibration Grid index to Frequency.
int GetFreqFromDetMagCalGrid(long index);
// Convert Phase Calibration Grid index to Frequency.
int GetFreqFromPhaseCalGrid(long index);	
int NormalizePhaseDegr(int phase);
double NormalizePhaseDegr(double phase);
float NormalizePhaseDegr(float phase);
// Check S21 receive measurements for evidence of broadband noise ingress
// due to unshielded fixture and local strong signal source
bool ExcessRxNoiseIngress(MeasurementSet *trace __gc[], int points);
